'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var uuid = require('node-uuid');

// Error constants
var CONFIG_REQUIRED = 'Configuration Object Required';
var PROCESS_REQUIRED = 'required paramenter [process] must be a function';
var SOURCE_REQUIRED = 'Source is required to be a function, promise or array';
var TYPE_PROCEED_ON_ERROR = 'parameter stopOnError must be a boolean';
var TYPE_EVENT_HANDLER = 'Event handlers must be functions';

var JobQueuer = function () {
  function JobQueuer(config) {
    _classCallCheck(this, JobQueuer);

    if (!config) throw new Error(CONFIG_REQUIRED);
    if (!config.process || typeof config.process !== 'function') throw new Error(PROCESS_REQUIRED);
    if (!config.source || typeof config.source !== 'function' && !Array.isArray(config.source) && !config.source.then) throw new Error(SOURCE_REQUIRED);
    if (config.stopOnError && typeof config.stopOnError !== 'boolean') throw new Error(TYPE_PROCEED_ON_ERROR);
    this.events = {};
    this.debug = config.debug;
    this.maxProceses = config.maxProceses || 1;
    this.process = config.process;
    this.stopOnError = config.stopOnError || false;
    this.sourceType = Array.isArray(config.source) ? 'array' : config.source.then ? 'promise' : 'function';
    if (this.sourceType === 'array') {
      this.source = config.source.slice(0);
    } else {
      this.source = config.source;
    }
    this.running = {};
    this.jobsFinished = 0;
    this.jobErrors = 0;
    this.fillingJobs = false;
    this.status = 'stoped';
  }

  _createClass(JobQueuer, [{
    key: 'on',
    value: function on(event, handler) {
      this.events[event] = handler;
      return this;
    }
  }, {
    key: 'emit',
    value: function emit(event, payload) {
      if (event === 'error' && this.stopOnError) this.status = 'error';
      if (this.debug && console) console.log('[' + new Date() + '][' + event + ']', payload);
      if (this.events[event]) this.events[event](payload);
    }
  }, {
    key: 'start',
    value: function start() {
      var _this = this;

      this.status = 'running';
      this.startTime = this.startTime || new Date();
      if (this.sourceType === 'promise') {
        var self = this;
        this.source.then(function (data) {
          _this.sourceType = Array.isArray(data) ? 'array' : data.then ? 'promise' : 'function';
          if (_this.sourceType === 'array') {
            _this.source = data.slice(0);
          } else {
            _this.source = data;
          }
          _this.start();
        }).catch(function (err) {
          _this.emit('error', err);
          _this.status = 'error';
          _this.emit('processFinish', {
            startTime: _this.startTime,
            endTime: new Date(),
            processed: _this.jobsFinished,
            errors: _this.jobErrors,
            status: _this.status
          });
        });
      } else {
        this.emit('start', {
          startTime: this.startTime,
          maxProceses: this.maxProceses,
          stopOnError: this.stopOnError,
          sourceType: this.sourceType,
          status: this.status,
          type: this.sourceType
        });
        this.fillJobs();
      }
      return this;
    }
  }, {
    key: 'runningJobsCount',
    value: function runningJobsCount() {
      return Object.keys(this.running).length;
    }
  }, {
    key: 'runJob',
    value: function runJob(jobPromise) {
      var _this2 = this;

      var jobId = uuid.v4();
      this.emit('jobRun', jobId);
      this.running[jobId] = jobPromise;
      var next = function next() {
        var self = _this2;
        var jobToDelete = _this2.running[jobId];
        delete _this2.running[jobId];
        var runningCount = Object.keys(_this2.running).length;
        if (!runningCount && _this2.status === 'empty' || _this2.status === 'error') {
          _this2.status = 'finished';
          return _this2.emit('processFinish', {
            startTime: _this2.startTime,
            endTime: new Date(),
            processed: _this2.jobsFinished,
            errors: _this2.jobErrors,
            status: _this2.status
          });
        }
        _this2.fillJobs();
      };

      var jobStartTime = new Date();
      jobPromise.then(function (result) {
        if (result) {
          var jobEndTime = new Date();
          _this2.emit('jobFinish', {
            jobId: jobId,
            jobStartTime: jobStartTime,
            jobEndTime: jobEndTime,
            result: result,
            jobsRunning: Object.keys(_this2.running).length
          });
          _this2.jobsFinished++;
        }
        next();
      }).catch(function (e) {
        _this2.emit('error', e);
        _this2.jobErrors++;
        next();
      });
    }
  }, {
    key: 'fillJobs',
    value: function fillJobs() {
      var _this3 = this;

      if (this.fillingJobs) return;
      this.fillingJobs = true;

      var resolveJobValue = function resolveJobValue(jobValue, resolve, reject) {
        if (jobValue) {
          var jobPromise = _this3.process(jobValue, function (err, value) {
            if (err) return reject(err);
            resolve(value);
          });
          if (jobPromise) {
            if (typeof jobPromise.then === 'function') return jobPromise.then(resolve);
            resolve(jobPromise);
          }
        } else {
          resolve();
          _this3.status = 'empty';
        }
      };

      while (Object.keys(this.running).length < this.maxProceses && this.status === 'running' && (this.sourceType === 'array' && this.source.length || this.sourceType !== 'array')) {
        this.emit('jobFetch', {
          jobsRunning: Object.keys(this.running).length
        });
        this.runJob(new Promise(function (resolve, reject) {
          var item = void 0;
          if (_this3.sourceType === 'array') {
            item = _this3.source.pop();
            if (!_this3.source.length) _this3.status = 'empty';
          } else {
            item = _this3.source(function (err, jobValue) {
              if (err) return reject(err);
              resolveJobValue(jobValue, resolve, reject);
            });
          }
          if (undefined !== item) {
            if (item && item.then && typeof item.then === 'function') {
              item.then(function (jobValue) {
                resolveJobValue(jobValue, resolve, reject);
              }).catch(reject);
            } else {
              if (item) {
                resolveJobValue(item, resolve, reject);
              } else {
                _this3.status = 'empty';
                resolve();
              }
            }
          }
        }));
      }
      this.fillingJobs = false;
    }
  }]);

  return JobQueuer;
}();

module.exports = JobQueuer;